;/*!common:static/utils/pageJump.js*/
/**
 * @file 演出首页
 * @desc data-url跳转
 * @example <div data-url="piaolist" bn-data={"a":"1"}></div>通过这种方式只能在本页面打开
 * @author  zhangyong06@baidu.com
 * @update YernSun(sunyueran)
 */

(function () {
    var JUMP_URL_ATTR = 'data-url';
    var JUMP_URL_DATA = 'bn-data';
    var JUMP_URL_TYPE = 'bn-target';
    var NA_URL = 'data-na';
    var wrapper = document.body;
    var clickEvent = 'ontouchend' in window ? 'touchend' : 'click';
    // 全局
    var t1 = null;
    //    document.addEventListener('gesturestart', function (e) {
    //        e.preventDefault();
    //    });
    // hack: 使active 伪类生效
    document.addEventListener('touchstart', function () {}, true);
    document.addEventListener('click', proxy, true);

    function matchSelector(element, selector) {
        if (!element || element.nodeType !== 1) {
            return false;
        }
        var parent;
        var match;
        var matchesSelector = element.webkitMatchesSelector || element.matchesSelector;
        if (matchesSelector) {
            match = matchesSelector.call(element, selector);
        }
        else {
            parent = element.parentNode;
            match = !!parent.querySelector(selector);
        }
        return match;
    }

    function navigate(url, bnData, bnTarget) {
        var args = arguments;
        bnData = JSON.parse(bnData || '{}');
        bnData.AppPage = url;
        if (url.indexOf('?') !== -1) {
            url.replace(/.+\?(.+)$/i, function ($0, query) {
                query.split('&').forEach(function (param) {
                    param = param.split('=');
                    bnData[param[0]] = param[1];
                });
            });
        }
        window.BNMJS.page.start('index', bnData, bnTarget);
    }

    function jsonToQuery(data) {
        var query = [];
        if (data) {
            for (var k in data) {
                if ((typeof data[k]).match(/number|string|boolean/)) {
                    query.push(encodeURIComponent(k) + '=' + encodeURIComponent(data[k]));
                }
            }
        }
        return query.join('&');
    }

    function proxy(e) {
        // 防止连续触发事件 例如设置了click为true的iscroll对象
        if (t1 == null) {
            t1 = new Date().getTime();
        }
        else {
            var t2 = new Date().getTime();
            if (t2 - t1 < 500) {
                t1 = t2;
                return;
            }
            /* eslint-disable no-else-return */
            else {
                t1 = t2;
            }
        }

        // 设置延时：发出stat统计
        setTimeout(function () {
            var element = e.target;
            var parent = element;
            var selector = '[data-url]';

            while (parent !== document.body) {
                if (matchSelector(parent, selector)) {
                    var bnData = parent.getAttribute(JUMP_URL_DATA);
                    var bnTarget = parent.getAttribute(JUMP_URL_TYPE);
                    var url = parent.getAttribute(NA_URL);
                    if (!url) {
                        url = parent.getAttribute(JUMP_URL_ATTR);
                    }
                    e.stopPropagation();
                    e.preventDefault();

                    // navigate(url, bn_data, bn_target);
                    navigate(url, bnData, bnTarget);
                    return;
                }
                else if (parent && parent.parentNode) {
                    parent = parent.parentNode;
                }
                else {
                    break;
                }
            }
        }, 100);
    }
})();
;/*!common:static/utils/widget.js*/
/**
 * @file Widget组件
 * @author Yern Sun <sunyueran@baidu.com>
 */
(function () {
    // 默认自动监听事件列表
    var eventsMap = [
        'click',
        'touchend',
        'touchstart',
        'change'
    ];

    function Widget(instance) {
        var self = this;
        // 组件事件列表
        self.$handle = {};
        var widgetID = new Date().getTime() % 10000000 + Math.random();

        /**
         * 生成事件阻止信号，同时避免事件监听被多次调用
         * @return {number} 组件事件阻止token
         */
        self.$preventEvent = (function () {
            var token = +new Date;
            return function () {
                return token;
            };
        })();
        // 可直接选择dom
        if (self.$el && self.$el.length) {
            self.$el = $(self.$el);
        }
        else if (self.name) {
            // 通过widget注册dom
            // 避免同名组件dom被多次监听及事件污染
            self.$el = $('[widget="' + self.name + '"]').not('[widget-id]').eq(0);
        }

        // 依据新实例注册事件，并且避免由于继承导致的多次事件监听
        if (!canRegisterEvent()) {
            return;
        }

        function canRegisterEvent() {
            return instance === 'instance' && self.$el && self.$el.size() && !self.$el.attr('widget-id');
        }
        self.$el.attr('widget-id', widgetID);
        // 自动监听事件
        eventsMap.forEach(function (event) {
            self.$el.on(event, '[data-' + event + ']', function (el) {
                el.stopPropagation();
                var action = $(this).data(event);
                var result;
                var params = $(this).data('params') || $(this).data();

                if (!params instanceof Object) {
                    try {
                        params = JSON.parse(params);
                    }
                    catch (e) {
                        console.warn($(this)[0], '参数格式错误');
                        params = {};
                    }
                }

                // 触发代理事件
                if ($.isFunction(self[action])) {
                    result = self[action].call(self, el, params);
                }
                else {
                    console.warn('组件:(' + self.name + ')方法' + action + '不存在');
                }
                // 提供事件截取的接口
                if (result === self.$preventEvent()) {
                    return;
                }
                // 触发外部接口对应的内部事件
                (self.$handle[action] || []).forEach(function (handle) {
                    if ($.isFunction(handle)) {
                        handle.call(self, result, params);
                    }
                });
            });
        });


        // 实现自定义初始化函数，避免多次执行，将初始化函数放在实例创建时
        //        if (self.init && $.isFunction(self.init)) {
        //            self.init.apply(self, arguments);
        //        }
    }



    /**
     * ajax异步方法
     * @return {Object} promise对象
     */
    Widget.prototype.ajax = function () {
        var dfd = $.Deferred();
        return dfd;
    };

    /**
     * 加载pagelet
     * @param   {string} name       [[Description]]
     * @param   {Object}   [option={] 请求option
     * @return {Object} promise对象
     */
    Widget.prototype.load = function (name, option) {
        var dfd = $.Deferred();
        var callback = function () {};
        option = option || {};

        if (typeof option.cb === 'function') {
            callback = option.cb;
        }
        window.BNMJS.page.getData(function (data) {
            if (option.param instanceof Object || option.params instanceof Object) {
                option.param = toForm($.extend({}, data, option.param || option.params)) || '';
            }
            if (!option.hideLoading) {
                $('#loading').removeClass('hide');
                setTimeout(function () {
                    if (!$('#loading').hasClass('hide')) {
                        $('#loading').addClass('hide');
                        window.BNMJS.ui.toast.show('请求超时！');
                    }
                }, 15000);
            }

            window.BigPipe.load($.extend(
                option, {
                    pagelets: [name],
                    cb: function () {
                        $('#loading').addClass('hide');
                        dfd.resolve.apply(dfd.resolve, arguments);
                        callback.apply(this, arguments);

                    }
                }));
        });
        return dfd;
    };


    /**
     * 析构函数，注销组件事件，资源回收
     */
    Widget.prototype.finalize = function () {
        var self = this;
        if (self.$el) {
            self.$el.off();
            for (var k in self) {
                self[k] = null;
                delete self[k];
            }
        }
    };

    /**
     * 注册引用事件
     * @param {string} event  事件名称
     * @param {function} handle 事件handle
     */
    Widget.prototype.on = function (event, handle) {
        if (!this.$handle[event]) {
            this.$handle[event] = [];
        }
        this.$handle[event].push(handle);
        return this;
    };

    /**
     * 注销引用事件
     * @param {string} event 事件名称
     */
    Widget.prototype.off = function (event) {
        if (this.$handle[event]) {
            delete this.$handle[event];
        }
    };

    Widget.prototype.onMessage = function () {

    };

    /**
     * 组件继承方法
     * @return {Object} Widget实例
     */
    Widget.extend = function (obj) {
        var Super = this;
        var subClass;
        var $super = new Super('extend');

        function SubClass() {
            Super.apply(this, arguments);
            this.$superInstance = $super;
        }
        // 实现继承
        SubClass.prototype = new Super('extend');

        $.extend(SubClass.prototype, obj);

        SubClass.extend = Super.extend;

        SubClass.new = function (params) {
            subClass = new this('instance');
            // 多重继承待测试
            subClass.$super = function (method, args) {
                if (method in $super && $.isFunction($super[method])) {
                    var superInstance = $super;
                    while (superInstance && superInstance[method] === this[method]) {
                        superInstance = $super.$superInstance;
                    }
                    if (superInstance && superInstance[method]) {
                        superInstance[method].apply(this, args);
                    }
                }
            };
            // 避免init继承的过程中被多次调用，如需调用父类构造函数，请手动调用$super
            if (subClass.init && $.isFunction(subClass.init)) {
                subClass.init.apply(subClass, arguments);
            }

            return subClass;
        };

        // subClass = new SubClass();
        // subClass.super = Super.prototype;

        return SubClass;
    };


    Widget.new = function (widget) {
        return;
    };

    window.Widget = Widget;

    /**
     * json转queryString，暂时不支持深层次对象
     * @param   {Object} [data={}] json对象
     * @return {string} queryString
     */
    function toForm(data) {
        var rs = [];
        data = data || {};
        for (var k in data) {
            if (data[k] === null || data[k] === undefined) {
                data[k] = '';
            }
            rs.push(encodeURIComponent(k) + '=' + encodeURIComponent(data[k]));
        }
        return rs.join('&');
    }
})();
;/*!common:static/utils/widget.page.js*/
/**
 * @file widget.page.js
 * @author Yern Sun <sunyueran@baidu.com>
 * @createDate 2016-11-15
 */

window.Widget.Page = window.Widget.extend({
    init: function () {
        var self = this;
        var args = arguments;
        // to avoid ready be called more then twice
        var readyCalled = false;
        if (self.$disableFilter === true) {
            $('#wrapper').css('margin-top', $('#header').height());
        }
        var name = self.$el.attr('widget');
        // 同步加载
        if (self.$el.find('#' + name).children().size() > 0) {
            self.ready.apply(self, args);
            readyCalled = true;
        }
        // pagelet加载
        else {
            window.BigPipe.on('pageletdone', function (option, request) {
                if (!readyCalled && request.id === self.name && $.isFunction(self.ready)) {
                    self.ready.apply(self, args);
                    readyCalled = true;
                }
            });
        }
        $('#wrapper').css({
            height: $(window).height() - $('#footer').height() - $('#header').height(),
            paddingTop: $('#header').height()
        });
    }
});
;/*!common:static/utils/ready.js*/
/**
 * @file ready.js
 * @author Yern Sun <sunyueran@baidu.com>
 * @createDate 2016-09-21
 * @description 该文件仅在糯米组件包中存在
 */
/* eslint-disable */
(function () {
    // 避免当前请求下重复加载resourceMap
    // 检测资源依赖

    window.BNJSReady = function (readyCallback, needLogin) {
//    debugger
//        alert('window.BNJS._isAllReady:' + window.BNJS);//._isAllReady);
        if (readyCallback && typeof readyCallback === 'function') {
            if (window.BNJS && typeof window.BNJS === 'object' && window.BNJS._isAllReady) {
                readyRender(readyCallback, needLogin);
            }
            else {
                document.addEventListener('BNJSReady', function () {
                    readyRender(readyCallback, needLogin);
                }, false);
            }
        }
    };

    function readyRender(readyCallback, needLogin) {
        if (!window.BNMJS) {
            if (window.location.origin && window.location.origin.indexOf('http') > -1) {
                window.ONLINEHOST = window.location.origin;
            }
            /**
 * @file bridge.js
 * @author Yern Sun <sunyueran@baidu.com>
 * @createDate 2016-09-05
 * {@link http://wiki.baidu.com/pages/viewpage.action?pageId=89199166&src=contextnavpagetreemode 文档参照BNJS}
 */
(function () {
    var BNJS = window.BNJS;
    function Bridge() {}
    // alert('BNJS == null ? ' + (BNJS == null));
    Bridge.prototype = copy(BNJS);
    function copy(obj) {
        var copyObj = {};
        if (obj instanceof Object) {
            for (var k in obj) {
                // alert(k);
                if (typeof obj[k] === 'object') {
                    copyObj[k] = copy(obj[k]);
                }
                else {
                    copyObj[k] = obj[k];
                }
            }
        }
        else {
            return {};
        }
        return copyObj;
    }

    /**
     * 设置糯米导航栏
     *
     * @param {Array} nav 导航列表
     * @param {number} 导航
     */
    Bridge.prototype.ui.title.setNav = function (nav, selectedIndex) {
        var titles = [];
        var links = [];
        var selectedIndex = selectedIndex || 0;
        if (nav instanceof Array) {
            nav.forEach(function (item, index) {
                titles.push(item.title || item.name);
                links.push(item.links);
                if (item.active || /active/i.test(item.className)) {
                    selectedIndex = index;
                }
            });
        }
        window.BNJS.ui.title.addTagList({
            id: 1,
            tagList: titles,
            selectIndex: selectedIndex,
            type: 2,
            callback: function (res) {
                var params = nav[res.data].params || {};
                params.AppPage = nav[res.data].link;
                window.BNMJS.page.jump(nav[res.data].link, params);
            }
        });
    };

    /**
     * [[Description]]
     * @param  {Object} [options={] [[Description]]
     * @return {Object} promise
     */
    Bridge.prototype.Ajax = function (options) {
        var dfd = $.Deferred();
        var url = options.url;
        if (window.BNJS.isMock) {
            return $.ajax(options);
        }
        var request = window.BNJS.http.get;
        options = options || {};
        if (options.type && options.type.toLowerCase() === 'post') {
            request = window.BNJS.http.post;
        }
        if (url.indexOf('http') !== 0) {
            url = window.ONLINEHOST + options.url;
        }
        request({
            url: url,
            params: options.data,
            respType: options.dataType,
            onSuccess: function (res) {
                dfd.resolve(res);
            },
            onFail: function (res) {
                dfd.reject(res);
            }
        });
        return dfd;
    };

    /**
     * 对BNJS.page.start进行二次封装并适配webapp
     * @param   {string}   url       URL值，可为相对路径
     * @param   {Object} params    跳转参数
     * @param   {number} action    打开方式
     * @param   {string} direction 打开方向
     * @return {string}   参见BNJS.page.start
     * @see http://wiki.baidu.com/display/baidunuomiwise/BNJS.page
     */
    Bridge.prototype.page.start = function (url, params, action, direction) {
        var args = arguments;
        var compId = BNJS.env.compId || 'piaofang';
        // set params
        args[1] = params || {};
        // 修正URL
        if (url.indexOf('http') === 0) {
            args[0] = 'bainuo://component?url=' + encodeURIComponent(args[0]);
        }
        else {
            // 自动修正tpl页面至组件的跳转
            args[0] = url.replace(/^(.+)$/i, function ($0, $1) {
                return 'bainuo://component?compid=' + compId + '&comppage=' + $1;
            });
        }
        return BNJS.page.start.apply(BNJS.page, args);
    };
    Bridge.prototype.page.getData = function (callback) {
        if (typeof callback === 'function') {
            callback(window._Params || {});
        }
    };

    Bridge.prototype.page.jump = function (url, params, action, direction) {
        params = params || {};
        params.AppPage = url;
        return BNJS.page.start(
            'bainuo://component?compid=piaofang&comppage=index',
            params,
            action,
            direction);
    };
    window.BNMJS = new Bridge();
})();;
        }
        window.BNJS.page.getData(function (data) {
            window._Params = data || {};
            readyCallback();
        });

    }
})();
;/*!common:static/utils/date.js*/
/**
 * @description 对Date的扩展，将 Date 转化为指定格式的String
 * @file date.js
 * @createDate 2016-09-06
 * @example (new Date()).Format('yyyy-MM-dd hh:mm:ss.S') ==> 2006-07-02 08:09:04.423 <br>
 * (new Date()).Format('yyyy-M-d h:m:s.S')      ==> 2006-7-2 8:9:4.18 
 */

(function () {
    window.Date.prototype.format = function (fmt) {
        var o = {
            'M+': this.getMonth() + 1, //月份         
            'd+': this.getDate(), //日         
            'h+': this.getHours() % 12 === 0 ? 12 : this.getHours() % 12, //小时         
            'H+': this.getHours(), //小时         
            'm+': this.getMinutes(), //分         
            's+': this.getSeconds(), //秒         
            'q+': Math.floor((this.getMonth() + 3) / 3), //季度         
            'S': this.getMilliseconds() //毫秒         
        };
        var week = {
            '0': '/u65e5',
            '1': '/u4e00',
            '2': '/u4e8c',
            '3': '/u4e09',
            '4': '/u56db',
            '5': '/u4e94',
            '6': '/u516d'
        };
        if (/(y+)/i.test(fmt)) {
            fmt = fmt.replace(RegExp.$1, (this.getFullYear() + '').substr(4 - RegExp.$1.length));
        }
        if (/(E+)/.test(fmt)) {
            fmt = fmt.replace(RegExp.$1, ((RegExp.$1.length > 1) ? (RegExp.$1.length > 2 ? '/u661f/u671f' : '/u5468') : '') + week[this.getDay() + '']);
        }
        for (var k in o) {
            if (new RegExp('(' + k + ')').test(fmt)) {
                fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
            }
        }
        return fmt;
    };

})();
;/*!common:static/utils/stat.js*/
/**
 * @file 统计逻辑
 * @author Yern Sun <sunyueran@baidu.com>
 * @createDate 2016-12-13
 */

(function (root) {
    if (!root.addStat) {
        var logImage = document.getElementById('stat-log');
        var url = 'https://movielog.baidu.com/img/transparent.gif';
        var container = document.getElementById('stat-container');

        function addStat(params, callback) {
            window.BNMJS.page.getData(function (data) {
                var defaultParams = {
                    'resid': 31, // 这个是刚接电影的时候留下来的，不知道还有没有用，写死的31
                    'from': data['from'] || 'zj', //  值只有webapp和zj，标识是手百或者组件
                    'da_ver': '2.1.0', // 迁移SAK平台，添加webapp标识
                    'da_trd': 'movie', // 写死movie, 表示是movie产品线的日志
                    'da_thirdpar': data['sfrom'] || '', // sfrom值
                    'da_subchannel': data['sub_channel'] || 'movie', // subchannel
                    'da_channel': data['da_channel'], // da_channel、da_client、da_module对应sfrom解析的channel、client、module
                    'da_client': data['da_client'],
                    'da_module': data['da_module']
                };
                var img = document.createElement('img');
                img.src = url + '?' + jsonToUrl($.extend(defaultParams, params));
                container.appendChild(img);
            });
        }
        addStat.$params = {};
        addStat.addParams = function (params) {
            return $.extend(this.$params, params || {});
        };


        $('body').on('click', '[data-log]', function (e) {
            var params = {};
            $.extend(params, this.$params, $(e.target).data(), {
                'da_act': 'click'
            });
        });
        window.addStat = addStat;

        function jsonToUrl(json) {
            if (!json) {
                return '';
            }
            var arr = [];
            for (var key in json) {
                if (json.hasOwnProperty(key)) {
                    if (json[key] === undefined || json[key] === null) {
                        json[key] = '';
                    }
                    arr.push(key + '=' + encodeURIComponent(json[key]));
                }
            }
            return arr.join('&');
        }
    }
})(window);