/**
 * @file ready.js
 * @author Yern Sun <sunyueran@baidu.com>
 * @createDate 2016-09-21
 * @description 该文件仅在糯米组件包中存在
 */
/* eslint-disable */
(function () {
    // 避免当前请求下重复加载resourceMap
    // 检测资源依赖

    window.BNJSReady = function (readyCallback, needLogin) {
        if (readyCallback && typeof readyCallback === 'function') {
            if (window.BNJS && typeof window.BNJS === 'object' && window.BNJS._isAllReady) {
                readyRender(readyCallback, needLogin);
            }
            else {
                document.addEventListener('BNJSReady', function () {
                    readyRender(readyCallback, needLogin);
                }, false);
            }
        }
    };

    function readyRender(readyCallback, needLogin) {
        if (!window.BNMJS) {
            if (window.location.origin && window.location.origin.indexOf('http') > -1) {
                window.ONLINEHOST = window.location.origin;
            }
            /**
 * @file bridge.js
 * @author Yern Sun <sunyueran@baidu.com>
 * @createDate 2016-09-05
 * {@link http://wiki.baidu.com/pages/viewpage.action?pageId=89199166&src=contextnavpagetreemode 文档参照BNJS}
 */
(function () {
    var BNJS = window.BNJS;

    function Bridge() {}
    Bridge.prototype = copy(BNJS);

    function copy(obj) {
        var copyObj = {};
        if (obj instanceof Object) {
            for (var k in obj) {
                if (typeof obj[k] === 'object') {
                    copyObj[k] = copy(obj[k]);
                }
                else {
                    copyObj[k] = obj[k];
                }
            }
        }
        else {
            return {};
        }
        return copyObj;
    }

    /**
     * 设置糯米导航栏
     *
     * @param {Array} nav 导航列表
     * @param {number} 导航
     */
    Bridge.prototype.ui.title.setNav = function (nav, selectedIndex) {
        var titles = [];
        var links = [];
        var selectedIndex = selectedIndex || 0;
        if (nav instanceof Array) {
            nav.forEach(function (item, index) {
                titles.push(item.title || item.name);
                links.push(item.links);
                if (item.active || /active/i.test(item.className)) {
                    selectedIndex = index;
                }
            });
        }
        window.BNJS.ui.title.addTagList({
            id: 1,
            tagList: titles,
            selectIndex: selectedIndex,
            type: 2,
            callback: function (res) {
                var params = nav[res.data].params || {};
                params.AppPage = nav[res.data].link;
                window.BNMJS.page.jump(nav[res.data].link, params);
            }
        });
    };

    /**
     * [[Description]]
     * @param  {Object} [options={] [[Description]]
     * @return {Object} promise
     */
    Bridge.prototype.Ajax = function (options) {
        var dfd = $.Deferred();
        var url = options.url;
        if (window.BNJS.isMock) {
            return $.ajax(options);
        }
        var request = window.BNJS.http.get;
        options = options || {};
        if (options.type && options.type.toLowerCase() === 'post') {
            request = window.BNJS.http.post;
        }
        if (url.indexOf('http') !== 0) {
            url = window.ONLINEHOST + options.url;
        }
        request({
            url: url,
            params: options.data,
            respType: options.dataType,
            onSuccess: function (res) {
                dfd.resolve(res);
            },
            onFail: function (res) {
                dfd.reject(res);
            }
        });
        return dfd;
    };

    /**
     * 对BNJS.page.start进行二次封装并适配webapp
     * @param   {string}   url       URL值，可为相对路径
     * @param   {Object} params    跳转参数
     * @param   {number} action    打开方式
     * @param   {string} direction 打开方向
     * @return {string}   参见BNJS.page.start
     * @see http://wiki.baidu.com/display/baidunuomiwise/BNJS.page
     */
    Bridge.prototype.page.start = function (url, params, action, direction) {
        var args = arguments;
        var compId = BNJS.env.compId || 'piaofang';
        // set params
        args[1] = params || {};
        // 修正URL
        if (url.indexOf('http') === 0) {
            args[0] = 'bainuo://component?url=' + encodeURIComponent(args[0]);
        }
        else {
            // 自动修正tpl页面至组件的跳转
            args[0] = url.replace(/^(.+)$/i, function ($0, $1) {
                return 'bainuo://component?compid=' + compId + '&comppage=' + $1;
            });
        }
        return BNJS.page.start.apply(BNJS.page, args);
    };
    Bridge.prototype.page.getData = function (callback) {
        if (typeof callback === 'function') {
            callback(window._Params || {});
        }
    };

    Bridge.prototype.page.jump = function (url, params, action, direction) {
        params = params || {};
        params.AppPage = url;
        return BNJS.page.start(
            'bainuo://component?compid=piaofang&comppage=index',
            params,
            action,
            direction);
    };
    window.BNMJS = new Bridge();
})();;
        }
        window.BNJS.page.getData(function (data) {
            window._Params = data || {};
            readyCallback();
        });

    }
})();