define('common:widget/calendar/calendar', function(require, exports, module) {

  /**
   * @file [[file]]
   * @author Yern Sun <sunyueran@baidu.com>
   * @createDate 2016-10-12
   * @description 日历组件
   */
  
  module.exports = require('common:widget/filter/filter').extend({
      name: 'common-datepicker',
      init: function () {
          var self = this;
          // 默认日期显示格式
          self.$format = 'YYYY年MM月dd日';
          self.$el.find('[widget="calendar-contaner"]').html($('[template="calendar"]').html());
          // 选中日期
          self.$pickedDate = new Date(this.$el.find('[data-init-date]').data('initDate') || +new Date);
  
          // 截至可选日期
          self.$endDay = this.$pickedDate;
          // 当前日期
          self.$today = new Date;
          self.$cycle = 'date';
      },
      confirm: function () {
          $('.mask').hide();
      },
      close: function (e) {
          if (e === true || $(e && e.target).data('click') === 'close') {
              this.$el.find('[widget="calendar"]').add('[widget="area"]').addClass('hide');
              this.$disbleScroll(false);
          }
          return this;
      },
  
      /**
       * 获取日期
       * @private
       * @param   {Object}   [picker={] [[Description]]
       * @return {[[Type]]} [[Description]]
       */
      _getDate: function (picker) {
          var params = {};
          picker = picker || {};
          if (!picker.date) {
              picker.date = this.$pickedDate;
          }
          if (picker.date) {
              params.date = picker.date.format('YYYY-MM-dd');
          }
          if (this.$cycle !== 'date') {
              params['end_date'] = picker['end_date'].format('YYYY-MM-dd');
          }
          return params;
      },
      changeMonth: function (el, params) {
          var self = this;
          var monthListContainer = self.$el.find('.month-list>ul');
          var total = monthListContainer.children().size();
          var monthDom = self.$el.find('[data-month-index]');
          var nextShowItem = monthDom.data('monthIndex') - params.diff;
          var translate = '';
          var changeMonthBtn = self.$el.find('[data-click="changeMonth"]');
          // 十二个月以内
          if (nextShowItem >= 0 && nextShowItem < total) {
              translate = 'translate(-' + (nextShowItem * $(window).width()) + 'px)';
              monthDom.attr('data-month-index', nextShowItem).html(
                  // 重置日历月份显示，由于索引是反向的，因而要减去1
                  monthListContainer.children().eq(total - nextShowItem - 1).data('month')
              );
              monthListContainer.css({
                  transform: translate
              });
              changeMonthBtn.removeClass('disable');
          }
  
          if (nextShowItem === 0) {
              changeMonthBtn.eq(1).addClass('disable');
          }
          else if (nextShowItem === total - 1) {
              changeMonthBtn.eq(0).addClass('disable');
          }
      },
  
      /**
       * 设置选中状态
       * 
       * @private
       * @return {[[Type]]} [[Description]]
       */
      _setActive: function () {
          if (this.$pickedDate) {
              var picked = this.$pickedDate.format('YYYY-MM-dd');
              this.$el.find('.active').removeClass('active');
              this.$el.find('[data-date="' + picked + '"]').addClass('active');
          }
          return this;
      },
  
      /**
       * 设置特殊日期，如节日等
       *
       * @protected
       * @return {Object} 值
       */
      $setSpecialDay: function () {
          var self = this;
          if (!self.specialDays) {
              // 初始化可选择日期
              self.specialDays = [self.$endDay.format('YYYY-MM-dd')];
              self.$el.find('[data-prediction]').each(function (k, dateDom) {
                  if ($(this).attr('clickable')) {
                      self.specialDays.push($(this).data('date'));
                  }
              });
          }
          return self.specialDays;
      },
  
      /**
       * 显示日历
       * @param {Object} e [[Description]]
       */
      showCalendar: function (e) {
          var self = this._setActive().close();
          var offset = self.$el.offset();
          var calendar = self.$el.find('[widget="calendar"]');
          $(e.target).addClass('on');
  
          calendar.toggleClass('hide').css({
              //            top: offset.top + offset.height
              top: offset.height,
              height: $(window).height() - $('#header').height()
          });
          (function () {
              var width = $(window).width();
              var li = self.$el.find('.date-list').css('width', width);
              var pickedMonth = self.$pickedDate.format('YYYY年MM月');
              var currentMonth = li.filter('[data-month="' + pickedMonth + '"]');
              // 性能待优化
              li.not(li.has('[clickable="1"]')).remove();
              li = li.has('[clickable="1"]');
  
              // 设置月份翻页点击按钮状态
              self.$setMonthPickerStatus(currentMonth);
  
              // 修复月份指针
              self.$el.find('[data-month-index]').html(currentMonth.data('month'))
                  .attr('data-month-index', li.size() - 1 - currentMonth.index());
              li.parent().css({
                  'width': width * li.size() + 'px',
                  'transform': 'translate(-' + ((li.size() - 1 - currentMonth.index()) * $(window).width()) + 'px)'
              });
              // 避免首次初始化时有动画
              setTimeout(function () {
                  li.parent().css({
                      transitionDuration: '0.5s'
                  });
              }, 10);
          })();
          self.$disbleScroll();
      },
      $setMonthPickerStatus: function (dom) {
          var picker = this.$el.find('.month-picker').children().removeClass('disable');
  
          if (!$(dom)[0].nextSibling) {
              picker.filter('.next-month').addClass('disable');
          }
          if (!$(dom)[0].previousSibling) {
              picker.filter('.prev-month').addClass('disable');
          }
      },
      /**
       * 选取日期
       * @param   {Object} e [[Description]]
       * @return {Object} [[Description]]
       */
      pickDate: function (e) {
          var dataDom = $(e.target);
          if (~~dataDom.attr('clickable') < 1) {
              return this.$preventEvent();
          }
          if (dataDom.data('date')) {
              e.preventDefault();
              // 避免选中无数据日期
              if (dataDom.data('date') > this.$today.format('YYYY-MM-dd') && !dataDom.data('prediction')) {
                  return this.$preventEvent();
              }
              this.$pickedDate = new Date(dataDom.data('date').replace(/-/g, '/'));
          }
          this.$el.find('.month-list').find('.active[data-date]').removeClass('active');
          this.$update().close(true);
          return {
              date: dataDom.addClass('active').data('date')
          };
      },
  
      /**
       * 选取日期
       * @param   {Object}   el [[Description]]
       * @return {Object} 提供外部事件所需参数
       */
      pick: function (el) {
          var self = this;
          var dayUnit = 86400000;
          var $el = $(el.target);
          var data = $el.data();
          var pickedDate = this.$pickedDate.format('YYYY-MM-dd');
          var specialDayList = this.$setSpecialDay();
          if (data.date && data.date.replace) {
              self.$pickedDate = new Date(data.date.replace(/-/g, '/'));
          }
          else if (data.diff) {
              var dayIndex = specialDayList.indexOf(pickedDate);
              // 特殊日期列表中含有当前日期
              if (specialDayList[dayIndex + ~~data.diff]) {
                  self.$pickedDate = new Date(specialDayList[dayIndex + ~~data.diff].replace(/-/g, '/'));
              }
              // 非特殊日期，前一天
              else if (data.diff < 0) {
                  var date = new Date(this.$pickedDate.getTime() + dayUnit * data.diff);
                  if (self.$getDateList().filter('[data-date="' + date.format('YYYY-MM-dd') + '"]').attr('clickable')) {
                      self.$pickedDate = date;
                  }
                  else {
                      return self.$preventEvent();
                  }
              }
              else {
                  return self.$preventEvent();
              }
          }
          return self.$update()._setActive()._getDate();
      },
  
      /**
       * @protected
       * @param   {string}   [date=this.$pickedDate] [[Description]]
       * @return {Object} 自身实例，提供链式调用
       */
      $update: function (date) {
          var showCalendarBtn = this.$el.find('[data-click="showCalendar"]');
          date = date || this.$pickedDate;
          if (date instanceof Date) {
              date = date.toLocaleDateString();
          }
          // 避免
          if (showCalendarBtn.size() < 1) {
              return this;
          }
          var matches = showCalendarBtn.html().match(/^(.*?)\d{4}(\D+)\d{1,2}(\D+?)\d{1,2}(\D+)$/);
          if (matches) {
              showCalendarBtn.html(date.replace(/^(\d{4})\D+(\d{1,2})\D+(\d{1,2})$/, function ($0, $1, $2, $3) {
                  return matches[1] + $1 + matches[2] + $2 + matches[3] + $3 + matches[4];
              }));
          }
  
          return this;
      },
  
      /**
       * 禁止页面滚动
       * @param {boolean} disable 是否禁用
       * @return {Object} this自身实例，提供链式调用
       */
      $disbleScroll: function (disable) {
          var method = 'toggleClass';
          if (disable === true) {
              method = 'addClass';
          }
          else if (disable === false) {
              method = 'removeClass';
          }
          if (!$('html, body, #wrapper')[method]('disable-scroll').hasClass('disable-scroll')) {
              this.$el.find('.toggle.on').removeClass('on');
          }
          return this;
      },
      $getDateList: function () {
          if (!this._dateList) {
              this._dateList = this.$el.find('.month-list>ul>li[data-month]').children('[data-date]:not([data-date=""])');
          }
          return this._dateList;
      }
  });

});
