define('common:widget/page/page', function(require, exports, module) {

  /**
   * @file page.js
   * @author Yern Sun <sunyueran@baidu.com>
   * @createDate 2016-11-30
   */
  
  var BNMJS = window.BNMJS;
  var Timer = window.Timer;
  
  var ONLINEHOST = window.ONLINEHOST;
  
  var homeNav = require('common:widget/home/home').new();
  
  
  window.now = window.now || new Date;
  
  function App() {
      var self = this;
      self.$container = $('[page-container]');
      self.$pages = {};
      self.$history = [];
  
      BNMJS.ui.showLoadingPage();
  
      // 初始化页面
      BNMJS.page.getData(function (data) {
          self.switchPage(data.AppPage || '/');
      });
  
  
      homeNav.on('switch', function (page) {
          self.switchPage(page.page);
      });
  
  
      // 重置返回按钮
      BNMJS.page.onBtnBackClick({
          callback: function () {
              if (self.$history.length === 1 || self.$history.pop() === '/') {
                  BNMJS.page.back();
              }
              else {
                  self.switchPage(self.$history.pop());
              }
          }
      });
  }
  
  App.prototype.setNavTitle = function (nav) {
      var self = this;
      var titles = [];
      var links = [];
      var selectedIndex = selectedIndex || 0;
      if (!nav) {
          self.$lastMenu = '';
          return BNMJS.ui.title.setClickableTitle('糯米票房');
      }
      $.each(nav || {}, function (index, item) {
          titles.push(item.title || item.name);
          links.push(item.links);
          if (item.active || /active/i.test(item.className)) {
              selectedIndex = index;
          }
      });
      var newMenu = '' + titles + links;
      if (self.$lastMenu !== newMenu) {
          self.$lastMenu = newMenu;
          window.BNJS.ui.title.addTagList({
              id: 1,
              tagList: titles,
              selectIndex: selectedIndex,
              //        type: 2,
              callback: function (res) {
                  self.switchPage(nav[res.data].link, nav[res.data].params);
              }
          });
      }
  };
  
  App.prototype.addPageStat = function (page) {
      var params = {
          'da_src': 'IndexPg',
          'da_act': 'ready'
      };
      if (page && page.length > 1) {
          params['da_src'] = page.replace(/^(.*?)(\?.*)?$/i, '$1').replace(/\/(\w)/ig, function ($0, $1) {
              return $1.toUpperCase();
          }) + 'Pg';
      }
      window.addStat(params);
  };
  
  App.prototype.switchPage = function (page) {
      var self = this;
      // 重置nav
      homeNav.setNav(page);
      var container = self.$container.add('#header-contaner').children().filter('[ container="' + page + '"]');
      self.$history.push(page);
  
      self.addPageStat(page);
  
      if (container.size() > 0) {
          self.setNavTitle(self.$pages[page].data.menu);
          container.removeClass('hide').siblings().addClass('hide');
          console.log('Switch to the tab ' + page + ' page has been loaded.');
      }
      else {
          $('.bainuo-load').removeClass('hide').siblings().addClass('hide');
  
          self.request(page)
              .then(function (res) {
                  self.$pages[page] = res;
                  self.render(page);
              }, function (res) {
                  console.log('页面加载出错:', res.responseURL);
                  if (res.status === 404) {
                      window.BNMJS.page.showErrorPage('页面不存在');
                  }
                  else {
                      window.BNMJS.page.showErrorPage('页面加载出错，请稍后重试！');
                  }
  
              });
      }
  };
  
  App.prototype.render = function (page) {
      var self = this;
      var res = self.$pages[page];
      var dfd = $.Deferred();
      Timer.load = +new Date;
      var css = $(res.data.css).filter('link[rel="stylesheet"]');
  
      // 标题
      self.setNavTitle(res.data.menu);
      if (res.data.filter) {
          var filter = $('<div container="' + page + '">' + res.data.filter + '</div>');
          $('#header-contaner').append(filter);
          filter.siblings().addClass('hide');
      }
  
      if (css.size() > 0) {
          // 避免闪屏现象，使HTML延迟在 CSS后加载
          css.eq(css.size() - 1).load(appendHTML);
      }
      else {
          appendHTML();
      }
  
      function appendHTML() {
          var content = $('<div container="' + page + '">' + res.data.content + '</div>');
          $('[page-container]').append(content);
          content.siblings().addClass('hide');
          BNMJS.ui.hideLoadingPage();
          self.log('CSS and html loaded cost:', Timer.load - Timer.ready, 'ms');
      }
  
      $('head').append(css);
  
      var scripts = $(res.data.script || '');
      var asyncScripts = scripts.filter('[src]');
      var counter = asyncScripts.size();
  
      // 加载依赖js
      asyncScripts.each(function () {
          var scriptTag = document.createElement('script');
          scriptTag.type = 'text/javascript';
          scriptTag.src = $(this).attr('src');
          scriptTag.onload = function () {
              if (--counter === 0) {
                  // 依赖js加载后
                  $('head').append(scripts.not(asyncScripts));
              }
          };
          document.head.appendChild(scriptTag);
      });
  
      return dfd;
  };
  
  App.prototype.request = function (page, params) {
      var self = this;
      var dfd = $.Deferred();
      params = params || {};
      if (this.$pages[page]) {
          dfd.resolve(self.$pages[page]);
      }
      else {
          dfd = window.BNMJS.Ajax({
              url: page,
              data: {
                  api: 'bainuo'
              }
          });
      }
      return dfd;
  };
  
  App.prototype.init = function (page) {
      var self = this;
      if (BNMJS.ui.title.setClickableTitle) {
          BNMJS.ui.title.setClickableTitle('糯米票房');
      }
      else {
          BNMJS.ui.title.setTitle('糯米票房');
      }
  
      if (!ONLINEHOST && window.location.hostname) {
          window.ONLINEHOST = ONLINEHOST = window.location.origin;
      }
  
      BNMJS.page.getData(function (data) {
          page = page || data.AppPage || '/';
          Timer.init = +new Date;
          self.log('First request:', Timer.init - Timer.ready, 'ms');
          if (page) {
              self.log(page);
          }
          self.load({
              url: page,
              data: {
                  api: 'bainuo'
              }
          }).fail(function (res) {
              BNMJS.ui.showErrorPage('页面加载失败！', 1);
          }).then(function (res) {
  
              var dfd = $.Deferred();
              Timer.load = +new Date;
              var css = $(res.data.css).filter('link[rel="stylesheet"]');
  
              if (res.data.menu) {
                  // 标题
                  BNMJS.ui.title.setNav(res.data.menu);
              }
  
              self.appendFilter(res.data.filter);
  
              // 避免闪屏现象，使HTML延迟在 CSS后加载
              css.eq(css.size() - 1).load(function () {
  
                  $('[page-container]').append(res.data.content);
                  BNMJS.ui.hideLoadingPage();
                  self.log('CSS and html loaded cost:', Timer.load - Timer.ready, 'ms');
              });
              $('head').append(css);
  
              var scripts = $(res.data.script || '');
              var asyncScripts = scripts.filter('[src]');
              var counter = asyncScripts.size();
  
              if (counter > 0) {
                  // 加载依赖js
                  asyncScripts.each(function () {
                      var scriptTag = document.createElement('script');
                      scriptTag.type = 'text/javascript';
                      scriptTag.src = $(this).attr('src');
                      scriptTag.onload = function () {
                          if (--counter === 0) {
                              // 依赖js加载后
                              dfd.resolve(scripts.not(asyncScripts));
                          }
                      };
                      document.head.appendChild(scriptTag);
                  });
              }
              else {
                  document.head.appendChild(scripts.not(asyncScripts));
              }
  
              return dfd;
          }).then(function (scripts) {
              $('head').append(scripts);
          });
      });
  };
  
  
  App.prototype.appendFilter = function (filter) {
      $('#header-contaner').html(filter || '');
  };
  
  App.prototype.log = function () {
      console.log.apply(console, arguments);
  };
  
  App.prototype.load = function (option) {
      return window.BNMJS.Ajax(option);
  };
  
  module.exports = function () {
      return new App();
  };

});
